/*
 * GkSu -- Gtk+ Frontend to su
 * Copyright (C) 2002 Gustavo Noronha Silva
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#include <gtk/gtk.h>

#include "defines.h"

#include "gksu.h"
#include "util.h"
#include "xauth.h"

/**
 * prepare_xauth:
 * @conf: the #GkConf to be filled with information about X
 * authorization stuff.
 *
 * Sets up the variables with values for the $DISPLAY 
 * environment variable and xauth-related stuff. Also
 * creates a temporary directory to hold a .Xauthority
 *
 * Returns: 0 if it suceeds.
 */
int
prepare_xauth (GkConf *conf)
{
  FILE *xauth_output;
  gchar xauth[256] = {0};

  gchar *tmp = NULL;
  gchar **tmpv = NULL;
  gchar *display = NULL;

  if (conf->ssh_fwd)
    {
      display = g_strdup (getenv ("DISPLAY"));
      tmpv = g_strsplit (display, ":", 3);
      g_free (display);

      conf->display = g_strdup_printf (":%s", tmpv[1]);

      g_strfreev (tmpv);
    }
  else
    conf->display = g_strdup (getenv ("DISPLAY"));

  /* get the authorization token */
  tmp = g_strdup_printf ("/usr/X11R6/bin/xauth list %s | "
			 "grep 'MIT-MAGIC-COOKIE-1' | "
			 "cut -d ' ' -f 5",
			 conf->display);
  xauth_output = popen (tmp, "r");
  fread (xauth, sizeof(char), 255, xauth_output);
  xauth[strlen(xauth) - 1] = '\0';
  pclose (xauth_output);
  g_free (tmp);

  conf->xauth = g_strdup (xauth);

  if (conf->debug)
    {
      fprintf(stderr, 
	      "xauth: -%s-\n"
	      "display: -%s-\n",
	      conf->xauth, conf->display);
    }

  return 0;
}

